{#- Error Output Assignments Template -#}
    // Error Output Assignments
    assign correctable_error = (error_state == ERR_DETECTED) && 
                               (parity_error || ecc_error) && !timeout_error;
    assign uncorrectable_error = (error_state == ERR_FATAL) || 
                                 (timeout_error || crc_error);
    assign error_code = {
        3'b0,
        error_state == ERR_FATAL,
        timeout_error,
        crc_error,
        parity_error,
        ecc_error
    };